import j from "jscodeshift";
import { format } from "prettier";

export const runPrettier = (source: string): string =>
  format(source, {
    trailingComma: "all",
    bracketSpacing: true,
    jsxBracketSameLine: false,
    semi: false,
    parser: "babel-ts",
  });

// https://github.com/benjamn/ast-types/blob/6e4d91213c27b6ba76a747c45200aa0e8498b176/src/scope.ts#L33C8-L33C28
export interface Scope {
  path: j.ASTPath;
  node: any;
  isGlobal: boolean;
  depth: number;
  parent: any;
  bindings: any;
  types: any;
  didScan: boolean;
  /** Returns whether the current scope declares a variable `name` */
  declares(name: string): boolean;
  /** Returns whether the current scope declares a variable `name` */
  declaresType(name: string): boolean;
  declareTemporary(prefix?: any): any;
  injectTemporary(identifier: any, init: any): any;
  scan(force?: any): any;
  /** Gets all the variables declared in the current scope */
  getBindings(): Record<
    string,
    Array<{
      value: j.ASTNode;
      node: j.ASTNode;
      parentPath: j.ASTPath;
    }>
  >;
  getTypes(): any;
  /** Starting from the current scope, work up and find the scope that declares the variable `name` */
  lookup(name: string): Scope;
  /** Starting from the current scope, work up and find the scope that declares the type `name` */
  lookupType(name: string): Scope;
  getGlobalScope(): Scope;
}
